@@ -33,6 +33,9 @@
 #include <errno.h>
 #include <libgen.h>
 
+#include <sys/types.h>
+#include <sys/stat.h>
+
 #include "nvidia-xconfig.h"
 #include "nvgetopt.h"
 
@@ -80,6 +83,35 @@ static void print_summary(void)
 #include "option_table.h"
 
 
+/*
+ * cook_description() - the description string may contain text within
+ * brackets, which is used by the manpage generator to denote text to
+ * be italicized.  We want to omit the bracket characters here.
+ */
+
+static char *cook_description(const char *description)
+{
+    int len;
+    char *s, *dst;
+    const char *src;
+    
+    len = strlen(description);
+    s = nvalloc(len + 1);
+    
+    for (src = description, dst = s; *src; src++) {
+        if (*src != '[' && (*src != ']')) {
+            *dst = *src;
+            dst++;
+        }
+    }
+
+    *dst = '\0';
+
+    return s;
+    
+} /* cook_description() */
+
+
 /*
  * print_help() - loop through the __options[] table, and print the
  * description of each option.
@@ -88,7 +120,7 @@ static void print_summary(void)
 static void print_help(int advanced)
 {
     int i, j, len;
-    char *msg, *tmp, scratch[64];
+    char *msg, *tmp, scratch[8], arg[64];
     const NVGetoptOption *o;
     
     print_version();
@@ -108,29 +140,51 @@ static void print_help(int advanced)
 
         if (!advanced && !(o->flags & OPTION_HELP_ALWAYS)) continue;
 
+        /* if we are going to need the argument, process it now */
+
+        if (o->flags & NVGETOPT_HAS_ARGUMENT) {
+            if (o->arg_name) {
+                strcpy(arg, o->arg_name);
+            } else {
+                len = strlen(o->name);
+                for (j = 0; j < len; j++) arg[j] = toupper(o->name[j]);
+                arg[len] = '\0';
+            }
+        }
+        
+        msg = nvstrcat("--", o->name, NULL);
+
         if (isalpha(o->val)) {
             sprintf(scratch, "%c", o->val);
-            msg = nvstrcat("-", scratch, ", --", o->name, NULL);
-        } else {
-            msg = nvstrcat("--", o->name, NULL);
-        }
-        if (o->flags & NVGETOPT_HAS_ARGUMENT) {
-            len = strlen(o->name);
-            for (j = 0; j < len; j++) scratch[j] = toupper(o->name[j]);
-            scratch[len] = '\0';
-            tmp = nvstrcat(msg, "=[", scratch, "]", NULL);
+
+            if (o->flags & NVGETOPT_HAS_ARGUMENT) {
+                tmp = nvstrcat("-", scratch, " ", arg, ", ", msg, NULL);
+            } else {
+                tmp = nvstrcat("-", scratch, ", ", msg, NULL);
+            }
             free(msg);
             msg = tmp;
         }
-        if (o->flags & NVGETOPT_IS_BOOLEAN ||
-            o->flags & NVGETOPT_IS_INTEGER) {
-            tmp = nvstrcat(msg, "/--no-", o->name, NULL);
+        
+        if (o->flags & NVGETOPT_HAS_ARGUMENT) {
+            tmp = nvstrcat(msg, "=", arg, NULL);
+            free(msg);
+            msg = tmp;
+        }
+        if (((o->flags & NVGETOPT_IS_BOOLEAN) &&
+             !(o->flags & NVGETOPT_HAS_ARGUMENT)) ||
+            (o->flags & NVGETOPT_ALLOW_DISABLE)) {
+            tmp = nvstrcat(msg, ", --no-", o->name, NULL);
             free(msg);
             msg = tmp;
         }
 
         fmtoutp(TAB, msg);
-        if (o->description) fmtoutp(BIGTAB, o->description);
+        if (o->description) {
+            tmp = cook_description(o->description);
+            fmtoutp(BIGTAB, tmp);
+            free(tmp);
+        }
         fmtout("");
         free(msg);
     }
@@ -143,44 +197,28 @@ static void print_help(int advanced)
  * and fill in any pertinent data from the commandline arguments
  */
 
-Options *parse_commandline(int argc, char *argv[])
+void parse_commandline(Options *op, int argc, char *argv[])
 {
-    Options *op;
     int c, boolval;
-    u32 bit;
     char *strval;
-    int intval;
+    int intval, disable;
 
-    op = (Options *) nvalloc(sizeof(Options));
-    
-    op->gop.x_project_root = "/usr/X11R6";
-    op->nvagp = -1;
-    op->digital_vibrance = -1;
-    op->transparent_index = -1;
-    op->stereo = -1;
     
     while (1) {
         
-        c = nvgetopt(argc, argv, __options, &strval, &boolval, &intval);
+        c = nvgetopt(argc, argv, __options, &strval,
+                     &boolval, &intval, &disable);
 
         if (c == -1)
             break;
 
         /* catch the boolean options */
 
-        if ((c >= XCONFIG_OPTION_START) &&
-            (c <= (XCONFIG_OPTION_START + XCONFIG_BOOL_OPTION_COUNT))) {
+        if ((c >= XCONFIG_BOOL_OPTION_START) &&
+            (c <= (XCONFIG_BOOL_OPTION_START + XCONFIG_BOOL_OPTION_COUNT))) {
+
+            set_boolean_option(op, c - XCONFIG_BOOL_OPTION_START, boolval);
             
-            bit = GET_BOOL_OPTION_BIT(c - XCONFIG_OPTION_START);
-            GET_BOOL_OPTION_SLOT(op->boolean_options,
-                                 c - XCONFIG_OPTION_START) |= bit;
-            if (boolval) {
-                GET_BOOL_OPTION_SLOT(op->boolean_option_values,
-                                     c - XCONFIG_OPTION_START) |= bit;
-            } else {
-                GET_BOOL_OPTION_SLOT(op->boolean_option_values,
-                                     c - XCONFIG_OPTION_START) &= ~bit;
-            }
             continue;
         }
         
@@ -197,7 +235,7 @@ Options *parse_commandline(int argc, char *argv[])
         case 'a': op->enable_all_gpus = TRUE; break;
         case '1': op->only_one_screen = TRUE; break;
         
-        case 'd': op->depth = atoi(strval);
+        case 'd': op->depth = intval;
             if ((op->depth != 8) &&
                 (op->depth != 15) &&
                 (op->depth != 16) &&
@@ -226,6 +264,14 @@ Options *parse_commandline(int argc, char *argv[])
         case FORCE_GENERATE_OPTION: op->force_generate = TRUE; break;
 
         case NVAGP_OPTION:
+
+            /* mark as disabled, so we can remove the option later */
+            
+            if (disable) {
+                op->nvagp = -2;
+                break;
+            }
+
             if      (strcasecmp(strval, "none")    == 0) op->nvagp = 0;
             else if (strcasecmp(strval, "nvagp")   == 0) op->nvagp = 1;
             else if (strcasecmp(strval, "agpgart") == 0) op->nvagp = 2;
@@ -242,18 +288,16 @@ Options *parse_commandline(int argc, char *argv[])
             }
             break;
 
-        case DIGITAL_VIBRANCE_OPTION:
-            if ( intval != -2 && (intval < 0 || intval > 255) ) {
-                fprintf(stderr, "\n");
-                fprintf(stderr, "Invalid digital vibrance: %d.\n", intval);
-                fprintf(stderr, "\n");
-                goto fail;
-            }
-            op->digital_vibrance = intval;
-            break;
-
         case TRANSPARENT_INDEX_OPTION:
-            if ( intval != -2 && (intval < 0 || intval > 255) ) {
+
+            /* mark as disabled, so we can remove the option later */
+            
+            if (disable) {
+                op->transparent_index = -2;
+                break;
+            }
+
+            if (intval < 0 || intval > 255) {
                 fprintf(stderr, "\n");
                 fprintf(stderr, "Invalid transparent index: %d.\n", intval);
                 fprintf(stderr, "\n");
@@ -263,7 +307,15 @@ Options *parse_commandline(int argc, char *argv[])
             break;
 
         case STEREO_OPTION:
-            if ( intval != -2 && (intval < 1 || intval > 6) ) {
+
+            /* mark as disabled, so we can remove the option later */
+            
+            if (disable) {
+                op->stereo = -2;
+                break;
+            }
+
+            if (intval < 1 || intval > 6) {
                 fprintf(stderr, "\n");
                 fprintf(stderr, "Invalid stereo: %d.\n", intval);
                 fprintf(stderr, "\n");
@@ -282,6 +334,51 @@ Options *parse_commandline(int argc, char *argv[])
             }
             break;
 
+        case REMOVE_MODE_OPTION:
+            nv_text_rows_append(&op->remove_modes, strval);
+            break;
+
+        case MULTI_GPU_OPTION:
+            {
+                const char* valid_values[] = {
+                    "0",
+                    "no",
+                    "off",
+                    "false",
+                    "single",
+                    "1",
+                    "yes",
+                    "on",
+                    "true",
+                    "auto",
+                    "afr",
+                    "sfr",
+                    "aa",
+                    NULL
+                };
+                int i;
+
+                /* mark as disabled, so we can remove the option later */
+                
+                if (disable) {
+                    op->multigpu = NV_DISABLE_STRING_OPTION;
+                    break;
+                }
+
+                for (i = 0; valid_values[i]; i++) {
+                    if (!strcasecmp(strval, valid_values[i]))
+                        break;
+                }
+
+                if (valid_values[i]) {
+                    op->multigpu = strval;
+                } else {
+                    fprintf(stderr, "Invalid MultiGPU option: %s.\n", strval);
+                    goto fail;
+                }
+            }
+            break;
+
         case SLI_OPTION:
             {
                 const char* valid_values[] = {
@@ -297,11 +394,19 @@ Options *parse_commandline(int argc, char *argv[])
                     "auto",
                     "afr",
                     "sfr",
-                    "sliaa",
+                    "aa",
+                    "afrofaa",
                     NULL
                 };
                 int i;
 
+                /* mark as disabled, so we can remove the option later */
+                
+                if (disable) {
+                    op->sli = NV_DISABLE_STRING_OPTION;
+                    break;
+                }
+
                 for (i = 0; valid_values[i]; i++) {
                     if (!strcasecmp(strval, valid_values[i]))
                         break;
@@ -332,6 +437,13 @@ Options *parse_commandline(int argc, char *argv[])
                 };
                 int i;
 
+                /* mark as disabled, so we can remove the option later */
+
+                if (disable) {
+                    op->rotate = NV_DISABLE_STRING_OPTION;
+                    break;
+                }
+
                 for (i = 0; valid_values[i]; i++) {
                     if (!strcasecmp(strval, valid_values[i]))
                         break;
@@ -348,6 +460,80 @@ Options *parse_commandline(int argc, char *argv[])
 
         case DISABLE_SCF_OPTION: op->disable_scf = TRUE; break;
         
+        case QUERY_GPU_INFO_OPTION: op->query_gpu_info = TRUE; break;
+
+        case 'E':
+            op->extract_edids_from_log = strval;
+            break;
+
+        case EXTRACT_EDIDS_OUTPUT_FILE_OPTION:
+            op->extract_edids_output_file = strval;
+            break;
+
+        case TWINVIEW_XINERAMA_INFO_ORDER_OPTION:
+            op->twinview_xinerama_info_order =
+                disable ? NV_DISABLE_STRING_OPTION : strval;
+            break;
+
+        case TWINVIEW_ORIENTATION_OPTION:
+            {
+                const char* valid_values[] = {
+                    "RightOf",
+                    "LeftOf",
+                    "Above",
+                    "Below",
+                    "Clone",
+                    NULL
+                };
+                int i;
+
+                if (disable) {
+                    op->twinview_orientation = NV_DISABLE_STRING_OPTION;
+                    break;
+                }
+                
+                for (i = 0; valid_values[i]; i++) {
+                    if (!strcasecmp(strval, valid_values[i]))
+                        break;
+                }
+
+                if (!valid_values[i]) {
+                    fprintf(stderr, "Invalid TwinViewOrientation option: "
+                            "\"%s\".\n", strval);
+                    goto fail;
+                }
+                
+                op->twinview_orientation = strval;
+            }
+            break;
+
+        case VIRTUAL_OPTION:
+            {
+                int ret, x, y;
+
+                if (disable) {
+                    op->virtual.x = op->virtual.y = -1;
+                    break;
+                }
+
+                ret = sscanf(strval, "%dx%d", &x, &y);
+                
+                if (ret != 2) {
+                    fprintf(stderr, "Invalid Virtual option: \"%s\".\n",
+                            strval);
+                    goto fail;
+                }
+                
+                op->virtual.x = x;
+                op->virtual.y = y;
+                
+                break;
+            }
+
+        case LOGO_PATH_OPTION:
+            op->logo_path = disable ? NV_DISABLE_STRING_OPTION : strval;
+            break;
+  
         default:
             goto fail;
         }
@@ -357,7 +543,40 @@ Options *parse_commandline(int argc, char *argv[])
     
     op->xconfig = tilde_expansion(op->xconfig);
     op->output_xconfig = tilde_expansion(op->output_xconfig);
+
+    return;
     
+ fail:
+    
+    fprintf(stderr, "\n");
+    fprintf(stderr, "Invalid commandline, please run `%s --help` "
+            "for usage information.\n", argv[0]);
+    fprintf(stderr, "\n");
+    exit(1);
+
+} /* parse_commandline() */
+
+
+
+/*
+ * load_default_options - malloc an Options structure
+ * and initialize it with default values.
+ *
+ */
+
+Options *load_default_options(void)
+{
+    Options *op;
+
+    op = (Options *) nvalloc(sizeof(Options));
+    if (!op) return NULL;
+    
+    op->nvagp = -1;
+    op->transparent_index = -1;
+    op->stereo = -1;
+
+    xconfigGenerateLoadDefaultOptions(&op->gop);
+
     /*
      * XXX save the option structure so that printing routines can
      * access it (and so that we don't have to carry the op everywhere
@@ -369,20 +588,9 @@ Options *parse_commandline(int argc, char *argv[])
         __op = op;
     }
     
-    return (op);
-    
- fail:
-    
-    fprintf(stderr, "\n");
-    fprintf(stderr, "Invalid commandline, please run `%s --help` "
-            "for usage information.\n", argv[0]);
-    fprintf(stderr, "\n");
-    exit(1);
-    
-    return NULL;
-
-} /* parse_commandline() */
+    return op;
 
+} /* load_default_options() */
 
 
 
@@ -527,6 +735,16 @@ int write_xconfig(Options *op, XConfigPtr config)
     fmtmsg("New X configuration file written to '%s'", filename);
     fmtmsg("");
     
+    
+    /* Set the default depth in the Solaris Management Facility 
+     * to the default depth of the first screen 
+     */
+    if (op->disable_scf == FALSE) {
+        if(!update_scf_depth(config->screens[0].defaultdepth)) {
+            goto done;
+        }
+    }
+    
     ret = TRUE;
     
  done:
@@ -620,7 +838,7 @@ static XConfigPtr find_system_xconfig(Options *op)
     
     /* Read the opened X config file */
     
-    error = xconfigReadConfigFile(op->screen, &config);
+    error = xconfigReadConfigFile(&config);
     if (error != XCONFIG_RETURN_SUCCESS) {
         xconfigCloseConfigFile();
         return NULL;;
@@ -630,6 +848,13 @@ static XConfigPtr find_system_xconfig(Options *op)
     
     xconfigCloseConfigFile();
     
+    /* Sanitize the X config file */
+    
+    if (!xconfigSanitizeConfig(config, op->screen, &(op->gop))) {
+        xconfigFreeConfig(config);
+        return NULL;
+    }
+
     return config;
 
 } /* find_system_xconfig() */
@@ -649,12 +874,6 @@ int update_xconfig(Options *op, XConfigPtr config)
         return FALSE;
     }
 
-    /* make sure the layout has the needed input devices */
-    
-    if (!xconfigCheckCoreInputDevices(config, layout)) {
-        return FALSE;
-    }
-
     /* apply multi-display options */
     
     if (!apply_multi_screen_options(op, config, layout)) {
@@ -679,6 +898,8 @@ int update_xconfig(Options *op, XConfigPtr config)
         }
     }
     
+    update_extensions(op, config);
+
     update_modules(config);
 
     update_banner(config);
@@ -688,53 +909,6 @@ int update_xconfig(Options *op, XConfigPtr config)
 } /* update_xconfig() */
 
 
-# define NV_LINE_LEN 1024
-static void get_xserver_in_use(Options *op)
-{
-#if defined(NV_SUNOS)    
-    op->gop.xserver=X_IS_XORG;
-#else
-    char *cmd;
-    FILE *stream = NULL;
-    int xserver = -1;
-
-    cmd = xconfigStrcat(op->gop.x_project_root, "/bin/X -version 2>&1", NULL);
-    if ((stream = popen(cmd, "r"))) {
-         char buf[NV_LINE_LEN];
-
-         while (1) {
-            if (fgets(buf, NV_LINE_LEN-1, stream) == NULL) break;
-            
-            if (xserver == -1) {
-                if (strcmp(buf, "XFree86") >= 0) {
-                    xserver = X_IS_XF86;
-                } else if (strcmp(buf, "X Window System") >= 0) {
-                    xserver = X_IS_XORG;
-                }
-            }
-        } 
-    }
-    /* Close the popen()'ed stream. */
-    pclose(stream);
-    free(cmd);
-
-    if (xserver == -1) {
-        char *xorgpath;
-
-        xorgpath = xconfigStrcat(op->gop.x_project_root, "/bin/Xorg", NULL);
-        if (access(xorgpath, F_OK)==0) {
-            xserver = X_IS_XORG;
-        } else {
-            xserver = X_IS_XF86;
-        }
-        free(xorgpath);
-    }
-    
-    op->gop.xserver=xserver;
-#endif
-} /* get_xserver_in_use */
-
-
 
 /*
  * main program entry point
@@ -753,9 +927,18 @@ int main(int argc, char *argv[])
     int ret;
     XConfigPtr config = NULL;
     
+
+    /* Load defaults */
+
+    op = load_default_options();
+    if (!op) {
+        fprintf(stderr, "\nOut of memory error.\n\n");
+        return 1;
+    }
+
     /* parse the commandline */
 
-    op = parse_commandline(argc, argv);
+    parse_commandline(op, argc, argv);
     
     /*
      * first, check for any of special options that cause us to exit
@@ -775,7 +958,17 @@ int main(int argc, char *argv[])
         xconfigGeneratePrintPossibleMice();
         return 0;
     }
+
+    if (op->query_gpu_info) {
+        ret = query_gpu_info(op);
+        return (ret ? 0 : 1);
+    }
  
+    if (op->extract_edids_from_log) {
+        ret = extract_edids(op);
+        return (ret ? 0 : 1);
+    }
+
     /*
      * we want to open and parse the system's existing X config file,
      * if possible
@@ -797,7 +990,7 @@ int main(int argc, char *argv[])
     /*
      * Get which X server is in use: Xorg or XFree86
      */
-    get_xserver_in_use(op);
+    xconfigGetXServerInUse(&op->gop);
     
     /*
      * if we failed to find the system's config file, generate a new
@@ -835,14 +1028,6 @@ int main(int argc, char *argv[])
         return 1;
     }
 
-    /* Set the default depth in the Solaris Management Facility 
-     * to the default depth of the first screen 
-     */
-    if (op->disable_scf == TRUE) {
-        return 0;
-    } else if(!update_scf_depth(config->screens[0].defaultdepth)) {
-        return 1;
-    }
     return 0;
     
 } /* main() */
